@@ -14,14 +14,11 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
 
 #include "gdm-dbus-util.h"
-#include <string.h>
-
-#include <glib/gstdio.h>
 #include <gio/gunixsocketaddress.h>
 
 /* a subset of org.freedesktop.DBus interface, to be used by internal servers */
@@ -71,34 +68,57 @@ handle_connection (GDBusServer      *server,
         return FALSE;
 }
 
+/* Note: Use abstract sockets like dbus does by default on Linux. Abstract
+ * sockets are only available on Linux.
+ */
+static char *
+generate_address (void)
+{
+        char *path;
+
+        if (g_unix_socket_address_abstract_names_supported ()) {
+                int   i;
+                char  tmp[9];
+
+                for (i = 0; i < 8; i++) {
+                        if (g_random_int_range (0, 2) == 0) {
+                                tmp[i] = g_random_int_range ('a', 'z' + 1);
+                        } else {
+                                tmp[i] = g_random_int_range ('A', 'Z' + 1);
+                        }
+                }
+                tmp[8] = '\0';
+
+                path = g_strdup_printf ("unix:abstract=/tmp/gdm-greeter-%s", tmp);
+        } else {
+                path = g_strdup ("unix:tmpdir=/tmp");
+        }
+
+        return path;
+}
+
 GDBusServer *
 gdm_dbus_setup_private_server (GDBusAuthObserver  *observer,
                                GError            **error)
 {
-        char *guid;
-        const char *client_address;
+        char *address, *guid;
         GDBusServer *server;
 
+        address = generate_address ();
         guid = g_dbus_generate_guid ();
 
-        server = g_dbus_server_new_sync ("unix:tmpdir=/tmp",
+        server = g_dbus_server_new_sync (address,
                                          G_DBUS_SERVER_FLAGS_NONE,
                                          guid,
                                          observer,
                                          NULL,
                                          error);
 
-        client_address = g_dbus_server_get_client_address (server);
-
-        if (g_str_has_prefix (client_address, "unix:path=")) {
-                client_address += strlen("unix:path=");
-                g_chmod (client_address, 0666);
-        }
-
         g_signal_connect (server, "new-connection",
                           G_CALLBACK (handle_connection),
                           NULL);
 
+        g_free (address);
         g_free (guid);
 
         return server;
