@@ -147,7 +147,7 @@ _gdm_server_query_ck_for_display_device (GdmServer *server)
         g_return_val_if_fail (GDM_IS_SERVER (server), NULL);
 
         error = NULL;
-        command = g_strdup_printf (LIBEXECDIR "/ck-get-x11-display-device --display %s",
+        command = g_strdup_printf (CONSOLEKIT_DIR "/ck-get-x11-display-device --display %s",
                                    server->priv->display_name);
 
         g_debug ("GdmServer: Running helper %s", command);
@@ -677,6 +677,8 @@ server_child_watch (GPid       pid,
                  : WIFSIGNALED (status) ? WTERMSIG (status)
                  : -1);
 
+        g_object_ref (server);
+
         if (WIFEXITED (status)) {
                 int code = WEXITSTATUS (status);
                 g_signal_emit (server, signals [EXITED], 0, code);
@@ -687,6 +689,8 @@ server_child_watch (GPid       pid,
 
         g_spawn_close_pid (server->priv->pid);
         server->priv->pid = -1;
+
+        g_object_unref (server);
 }
 
 static gboolean
@@ -774,9 +778,17 @@ gdm_server_start (GdmServer *server)
         const char *vtarg = NULL;
 
         /* Hardcode the VT for the initial X server, but nothing else */
-        if (server->priv->is_initial
-            && g_strcmp0 (server->priv->display_seat_id, "seat0") == 0) {
+        if (server->priv->is_initial) {
                 vtarg = "vt" GDM_INITIAL_VT;
+
+#ifdef WITH_SYSTEMD
+                /* undo the hardcoding if we are an auxillary seat */
+                if (sd_booted () > 0) {
+                     if (strcmp (server->priv->display_seat_id, "seat0") != 0) {
+                         vtarg = NULL;
+                     }
+                }
+#endif
         }
 
         /* fork X server process */
