@@ -64,7 +64,7 @@
 #define MAX_CONNECT_ATTEMPTS  10
 #define DEFAULT_PING_INTERVAL 15
 
-#define INITIAL_SETUP_USERNAME "gdm-initial-setup"
+#define INITIAL_SETUP_USERNAME "gnome-initial-setup"
 
 struct GdmSimpleSlavePrivate
 {
@@ -107,6 +107,13 @@ enum {
 static void     gdm_simple_slave_class_init     (GdmSimpleSlaveClass *klass);
 static void     gdm_simple_slave_init           (GdmSimpleSlave      *simple_slave);
 static void     gdm_simple_slave_finalize       (GObject             *object);
+static void     gdm_simple_slave_open_reauthentication_channel (GdmSlave             *slave,
+                                                                const char           *username,
+                                                                GPid                  pid_of_caller,
+                                                                uid_t                 uid_of_caller,
+                                                                GAsyncReadyCallback   callback,
+                                                                gpointer              user_data,
+                                                                GCancellable         *cancellable);
 
 G_DEFINE_TYPE (GdmSimpleSlave, gdm_simple_slave, GDM_TYPE_SLAVE)
 
@@ -226,9 +233,7 @@ on_session_exited (GdmSession       *session,
         g_object_set (GDM_SLAVE (slave), "session-id", NULL, NULL);
 
         g_debug ("GdmSimpleSlave: session exited with code %d\n", exit_code);
-        if (slave->priv->start_session_service_name == NULL) {
-                gdm_slave_stopped (GDM_SLAVE (slave));
-        }
+        gdm_slave_stop (GDM_SLAVE (slave));
 }
 
 static void
@@ -242,10 +247,7 @@ on_session_died (GdmSession       *session,
         g_debug ("GdmSimpleSlave: session died with signal %d, (%s)",
                  signal_number,
                  g_strsignal (signal_number));
-
-        if (slave->priv->start_session_service_name == NULL) {
-                gdm_slave_stopped (GDM_SLAVE (slave));
-        }
+        gdm_slave_stop (GDM_SLAVE (slave));
 }
 
 static gboolean
@@ -328,6 +330,7 @@ static void
 stop_greeter (GdmSimpleSlave *slave)
 {
         char *username;
+        gboolean script_successful;
 
         g_debug ("GdmSimpleSlave: Stopping greeter");
 
@@ -343,10 +346,20 @@ stop_greeter (GdmSimpleSlave *slave)
         }
 
         if (username != NULL) {
-                gdm_slave_run_script (GDM_SLAVE (slave), GDMCONFDIR "/PostLogin", username);
+                script_successful = gdm_slave_run_script (GDM_SLAVE (slave), GDMCONFDIR "/PostLogin", username);
+        } else {
+                script_successful = TRUE;
         }
         g_free (username);
 
+        if (!script_successful) {
+                g_debug ("GdmSimpleSlave: PostLogin script unsuccessful");
+
+                slave->priv->start_session_id = 0;
+                queue_greeter_reset (slave);
+                return;
+        }
+
         gdm_launch_environment_stop (GDM_LAUNCH_ENVIRONMENT (slave->priv->greeter_environment));
 }
 
@@ -521,6 +534,9 @@ on_session_reauthentication_started (GdmSession      *session,
                                                            g_free);
                 g_simple_async_result_complete_in_idle (result);
         }
+
+        g_hash_table_remove (slave->priv->open_reauthentication_requests,
+                             source_tag);
 }
 
 static void
@@ -685,7 +701,7 @@ on_session_client_disconnected (GdmSession          *session,
                       NULL);
 
         if ( ! display_is_local && !slave->priv->session_is_running) {
-                gdm_slave_stopped (GDM_SLAVE (slave));
+                gdm_slave_stop (GDM_SLAVE (slave));
         }
 }
 
@@ -825,7 +841,7 @@ on_greeter_environment_session_stopped (GdmLaunchEnvironment *greeter_environmen
 {
         g_debug ("GdmSimpleSlave: Greeter stopped");
         if (slave->priv->start_session_service_name == NULL) {
-                gdm_slave_stopped (GDM_SLAVE (slave));
+                gdm_slave_stop (GDM_SLAVE (slave));
         } else {
                 start_session (slave);
         }
@@ -841,7 +857,7 @@ on_greeter_environment_session_exited (GdmLaunchEnvironment    *greeter_environm
 {
         g_debug ("GdmSimpleSlave: Greeter exited: %d", code);
         if (slave->priv->start_session_service_name == NULL) {
-                gdm_slave_stopped (GDM_SLAVE (slave));
+                gdm_slave_stop (GDM_SLAVE (slave));
         }
 }
 
@@ -852,7 +868,7 @@ on_greeter_environment_session_died (GdmLaunchEnvironment    *greeter_environmen
 {
         g_debug ("GdmSimpleSlave: Greeter died: %d", signal);
         if (slave->priv->start_session_service_name == NULL) {
-                gdm_slave_stopped (GDM_SLAVE (slave));
+                gdm_slave_stop (GDM_SLAVE (slave));
         }
 }
 
@@ -933,8 +949,6 @@ setup_server (GdmSimpleSlave *slave)
         /* Set the busy cursor */
         gdm_slave_set_busy_cursor (GDM_SLAVE (slave));
 
-        /* Set the initial keyboard layout to something reasonable */
-        gdm_slave_set_initial_keyboard_layout (GDM_SLAVE (slave));
         /* The root window has a background that may be useful
          * to cross fade or transition from when setting the
          * login screen background.  We read it here, and stuff
@@ -1056,7 +1070,7 @@ start_launch_environment (GdmSimpleSlave *slave,
 
         g_debug ("GdmSimpleSlave: Creating greeter on %s %s %s", display_name, display_device, display_hostname);
         slave->priv->greeter_environment = create_environment (session_id,
-                                                               GDM_USERNAME,
+                                                               username,
                                                                display_name,
                                                                seat_id,
                                                                display_device,
@@ -1111,23 +1125,21 @@ create_initial_setup_user (GdmSimpleSlave *slave)
         gboolean ret = TRUE;
         ActUserManager *act;
         ActUser *user;
-        const char *username;
         GFile *src_file, *dest_file;
         GError *error = NULL;
         const char *e = NULL;
 
-        username = "gnome-initial-setup";
-
         /* First, create the user */
         act = act_user_manager_get_default ();
 
-        user = act_user_manager_create_user (act, username, "", 0, &error);
+        user = act_user_manager_create_user (act, INITIAL_SETUP_USERNAME, "", 0, &error);
         if (user == NULL) {
                 if (g_dbus_error_is_remote_error (error)) {
                         e = g_dbus_error_get_remote_error (error);
 		}
 
-                g_warning ("Creating user '%s' failed: %s / %s", username, e, error->message);
+                g_warning ("Creating user '%s' failed: %s / %s",
+                           INITIAL_SETUP_USERNAME, e, error->message);
 
                 if (g_strcmp0 (e, "org.freedesktop.Accounts.Error.UserExists") != 0) {
                         ret = FALSE;
@@ -1168,11 +1180,9 @@ destroy_initial_setup_user (GdmSimpleSlave *slave)
 {
         ActUserManager *act;
         ActUser *user;
-        const char *username;
         const char *filename;
         GError *error;
 
-        username = "gnome-initial-setup";
         filename = RULES_DIR RULES_FILE;
 
         if (g_remove (filename) < 0) {
@@ -1182,9 +1192,9 @@ destroy_initial_setup_user (GdmSimpleSlave *slave)
         act = act_user_manager_get_default ();
 
         error = NULL;
-        user = act_user_manager_get_user (act, username);
+        user = act_user_manager_get_user (act, INITIAL_SETUP_USERNAME);
         if (!act_user_manager_delete_user (act, user, TRUE, &error)) {
-                g_warning ("Failed to create user '%s': %s", username, error->message);
+                g_warning ("Failed to delete user '%s': %s", INITIAL_SETUP_USERNAME, error->message);
                 g_error_free (error);
         }
 
@@ -1210,7 +1220,7 @@ wants_autologin (GdmSimpleSlave *slave)
         return enabled && delay > 0;
 }
 
-#define INITIAL_SETUP_TRIGGER_FILE LOCALSTATEDIR "lib/gdm/run-initial-setup"
+#define INITIAL_SETUP_TRIGGER_FILE LOCALSTATEDIR "/lib/gdm/run-initial-setup"
 
 static gboolean
 wants_initial_setup (GdmSimpleSlave *slave)
@@ -1277,7 +1287,7 @@ on_server_exited (GdmServer      *server,
 {
         g_debug ("GdmSimpleSlave: server exited with code %d\n", exit_code);
 
-        gdm_slave_stopped (GDM_SLAVE (slave));
+        gdm_slave_stop (GDM_SLAVE (slave));
 
 #ifdef WITH_PLYMOUTH
         if (slave->priv->plymouth_is_running) {
@@ -1295,7 +1305,7 @@ on_server_died (GdmServer      *server,
                  signal_number,
                  g_strsignal (signal_number));
 
-        gdm_slave_stopped (GDM_SLAVE (slave));
+        gdm_slave_stop (GDM_SLAVE (slave));
 
 #ifdef WITH_PLYMOUTH
         if (slave->priv->plymouth_is_running) {
@@ -1426,12 +1436,10 @@ gdm_simple_slave_open_reauthentication_channel_finish (GdmSlave      *slave,
 {
         GdmSimpleSlave  *self = GDM_SIMPLE_SLAVE (slave);
         const char      *address;
-        GPid             pid_of_caller;
 
-        pid_of_caller = GPOINTER_TO_INT (g_simple_async_result_get_source_tag (G_SIMPLE_ASYNC_RESULT (result)));
-
-        g_hash_table_remove (self->priv->open_reauthentication_requests,
-                             GINT_TO_POINTER (pid_of_caller));
+        g_return_val_if_fail (g_simple_async_result_is_valid (result,
+                                                              G_OBJECT (self),
+                                                              gdm_simple_slave_open_reauthentication_channel), NULL);
 
         address = g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (result));
 
@@ -1457,14 +1465,10 @@ gdm_simple_slave_open_reauthentication_channel (GdmSlave             *slave,
         result = g_simple_async_result_new (G_OBJECT (slave),
                                             callback,
                                             user_data,
-                                            GINT_TO_POINTER (pid_of_caller));
+                                            gdm_simple_slave_open_reauthentication_channel);
 
         g_simple_async_result_set_check_cancellable (result, cancellable);
 
-        g_hash_table_insert (self->priv->open_reauthentication_requests,
-                             GINT_TO_POINTER (pid_of_caller),
-                             g_object_ref (result));
-
         if (!self->priv->session_is_running) {
                 g_simple_async_result_set_error (result,
                                                  G_DBUS_ERROR,
@@ -1473,10 +1477,16 @@ gdm_simple_slave_open_reauthentication_channel (GdmSlave             *slave,
                 g_simple_async_result_complete_in_idle (result);
 
         } else {
+                g_hash_table_insert (self->priv->open_reauthentication_requests,
+                                     GINT_TO_POINTER (pid_of_caller),
+                                     g_object_ref (result));
+
                 gdm_session_start_reauthentication (self->priv->session,
                                                     pid_of_caller,
                                                     uid_of_caller);
         }
+
+        g_object_unref (result);
 }
 
 static gboolean
@@ -1500,6 +1510,7 @@ gdm_simple_slave_stop (GdmSlave *slave)
 
         if (self->priv->greeter_environment != NULL) {
                 stop_greeter (self);
+                self->priv->greeter_environment = NULL;
         }
 
         if (self->priv->session_is_running) {
@@ -1518,6 +1529,7 @@ gdm_simple_slave_stop (GdmSlave *slave)
                 gdm_simple_slave_revoke_console_permissions (self);
 #endif
 
+                self->priv->session_is_running = FALSE;
         }
 
         if (self->priv->session != NULL) {
@@ -1578,7 +1590,7 @@ gdm_simple_slave_finalize (GObject *object)
 
         g_return_if_fail (slave->priv != NULL);
 
-        gdm_simple_slave_stop (GDM_SLAVE (slave));
+        gdm_slave_stop (GDM_SLAVE (slave));
 
         g_hash_table_unref (slave->priv->open_reauthentication_requests);
 
