@@ -44,10 +44,6 @@
 #include <X11/extensions/Xrandr.h>
 #include <X11/Xatom.h>
 
-#ifdef HAVE_LIBXKLAVIER
-#include <libxklavier/xklavier.h>
-#endif
-
 #ifdef WITH_SYSTEMD
 #include <systemd/sd-login.h>
 #include <systemd/sd-daemon.h>
@@ -368,9 +364,7 @@ gdm_slave_run_script (GdmSlave   *slave,
 
         if (WIFEXITED (status)) {
                 g_debug ("GdmSlave: Process exit status: %d", WEXITSTATUS (status));
-                ret = WEXITSTATUS (status) != 0;
-        } else {
-                ret = TRUE;
+                ret = WEXITSTATUS (status) == 0;
         }
 
  out:
@@ -450,51 +444,6 @@ gdm_slave_save_root_windows (GdmSlave *slave)
         XSync (slave->priv->server_display, False);
 }
 
-void
-gdm_slave_set_initial_keyboard_layout (GdmSlave *slave)
-{
-#ifdef HAVE_LIBXKLAVIER
-        XklEngine    *engine;
-        XklConfigRec *config;
-
-        engine = xkl_engine_get_instance (slave->priv->server_display);
-        config = xkl_config_rec_new ();
-
-        if (xkl_config_rec_get_from_server (config, engine)) {
-                int i;
-
-                for (i = 1; config->layouts[i] != NULL; i++) {
-                        /* put us at the front of the list, since usernames and
-                         * passwords are usually ascii
-                         */
-                        if (strcmp (config->layouts[i], "us") == 0) {
-                                char *temp_layout;
-                                char *temp_variant = NULL;
-                                char *temp_options = NULL;
-
-                                temp_layout = config->layouts[0];
-                                config->layouts[0] = config->layouts[i];
-                                config->layouts[i] = temp_layout;
-
-                                if (config->variants != NULL) {
-                                        temp_variant = config->variants[0];
-                                        config->variants[0] = config->variants[i];
-                                        config->variants[i] = temp_variant;
-                                }
-
-                                if (config->options != NULL) {
-                                        temp_options = config->options[0];
-                                        config->options[0] = config->options[i];
-                                        config->options[i] = temp_options;
-                                }
-                                break;
-                        }
-                }
-                xkl_config_rec_activate (config, engine);
-        }
-#endif
-}
-
 static void
 determine_initial_cursor_position (GdmSlave *slave,
                                    int      *x,
@@ -947,18 +896,12 @@ gdm_slave_stop (GdmSlave *slave)
         g_debug ("GdmSlave: stopping slave");
 
         g_object_ref (slave);
+
         ret = GDM_SLAVE_GET_CLASS (slave)->stop (slave);
-        g_object_unref (slave);
-
-        return ret;
-}
-
-void
-gdm_slave_stopped (GdmSlave *slave)
-{
-        g_return_if_fail (GDM_IS_SLAVE (slave));
-
         g_signal_emit (slave, signals [STOPPED], 0);
+
+        g_object_unref (slave);
+        return ret;
 }
 
 gboolean
@@ -1551,7 +1494,7 @@ activate_session_id_for_systemd (GdmSlave   *slave,
                                              NULL,
                                              &error);
         if (reply == NULL) {
-                g_debug ("GdmSlave: logind %s raised:\n %s\n\n",
+                g_debug ("GdmSlave: logind 'ActivateSessionOnSeat' %s raised:\n %s\n\n",
                          g_dbus_error_get_remote_error (error), error->message);
                 g_error_free (error);
                 return FALSE;
@@ -1635,7 +1578,7 @@ session_unlock_for_systemd (GdmSlave   *slave,
                                              NULL,
                                              &error);
         if (reply == NULL) {
-                g_debug ("GdmSlave: logind %s raised:\n %s\n\n",
+                g_debug ("GdmSlave: logind 'UnlockSession' %s raised:\n %s\n\n",
                          g_dbus_error_get_remote_error (error), error->message);
                 g_error_free (error);
                 return FALSE;
